import argparse
import json
import os
import requests

from .config_helper import PolybarVikunjaConfig
from .popups import ConfigPopup, RemainingTodosPopup


class PolybarVikunjaClient():

    def __init__(self, *args, **kwargs):
        '''
        :kwarg config_file: String path relative to ~/ of a config file to load.
        '''
        self.config_file = kwargs.get(
            'config_file',
            '.polybar_vikunja_config.json'
        )
        self.config = PolybarVikunjaConfig(config_file=self.config_file)

        self.base_url = self.config.get(
            'base_url',
            kwargs.get('base_url', None)
        )
        self.username = self.config.get(
            'username',
            kwargs.get('username', None)
        )
        self.password = self.config.get(
            'password',
            kwargs.get('password', None)
        )
        self.default_list = self.config.get(
            "default_list",
            1
        )

        self.login()

    def login(self, refresh_token=False):
        '''
        Logs in to the Vikunja instance using your config.
        '''
        self.jwt = self.config.get('jwt', None)

        if not self.jwt or refresh_token:
            login_url = self.base_url + '/api/v1/login'
            response = requests.post(
                login_url,
                headers={
                    "Content-Type": "application/json"
                },
                json={
                    "username": self.username,
                    "password": self.password
                }
            )

            self.jwt = response.json()["token"]
            self.config.set("jwt", self.jwt)

    def list_todo_lists(self):
        '''
        Returns a dict of list names to IDs.
        '''
        response = requests.get(
            self.base_url + "/api/v1/lists",
            headers={
                "Authorization": f"Bearer {self.jwt}"
            }
        )

        response = response.json()
        if "message" in response and response["message"] == "invalid or expired jwt":
            self.login(refresh_token=True)
            return self.list_todo_lists()
        else:
            return response

    def list_list_contents(self):
        response = requests.get(
            self.base_url + f"/api/v1/lists/{self.default_list}/tasks",
            headers={
                "Authorization": f"Bearer {self.jwt}"
            }
        )

        response = response.json()
        if "message" in response and response["message"] == "invalid or expired jwt":
            self.login(refresh_token=True)
            return self.list_list_contents()
        else:
            return response

    def get_remaining_todos(self):
        response = requests.get(
            self.base_url + f"/api/v1/lists/{self.default_list}/tasks",
            headers={
                "Authorization": f"Bearer {self.jwt}"
            },
            params={
                "filter_by": "done",
                "filter_value": "false",
                "filter_comparator": "equals"
            }
        )

        response = response.json()
        if "message" in response and response["message"] == "invalid or expired jwt":
            self.login(refresh_token=True)
            return self.get_remaining_todos()
        else:
            return response


    def get_todo_count(self):
        return len(self.get_remaining_todos())

    def mark_todo_complete_status(self, todo_id, is_complete):
        response = requests.post(
            self.base_url + f"/api/v1/tasks/{todo_id}",
            headers={
                "Authorization": f"Bearer {self.jwt}"
            },
            json={
                "done": True,
            }
        )

        response = response.json()
        if "message" in response and response["message"] == "invalid or expired jwt":
            self.login(refresh_token=True)
            return self.mark_todo_complete_status(todo_id, is_complete)
        else:
            return response


def first_run():
    '''
    Runs the first time polybar-vikunja is initialized. Gives a basic
    interactive menu and asks the user for details to init the config file.
    '''
    print("\tWelcome to polybar-vikunja!")
    print("This script will set up the initial config file for you.")
    print("WARNING: This script *will* store your credentials in plaintext!")

    config = {}

    response = input("Vikunja Base URL: (e.g. https://try.vikunja.io) ")
    config['base_url'] = response

    response = input("Username: ")
    config['username'] = response

    response = input("Password: ")
    config['password'] = response

    client = PolybarVikunjaClient(
        base_url=config['base_url'],
        username=config['username'],
        password=config['password']
    )

    try:
        client.login()
    except Exception as e:
        print("Problem with your config, try again!")
        print(e)
        first_run()

        return

    # Now set the default list:
    lists = {
        e["title"]: e["id"] for e in client.list_todo_lists()
    }

    print("Lists:")
    for title in lists:
        print(f"\t{title} ({lists[title]})")

    list_id = input("What list number would you like to watch? ")
    config["default_list"] = str(list_id)

    try:
        config_file_dir = client.config_file
        path = os.path.expanduser("~") + f"/{config_file_dir}"
        with open(path, 'w') as fh:
            json.dump(config, fh)
    except Exception as e:
        print(f"Problem writing config file:")
        print(e)

        print("Here's your config to manually add:")
        print(config)

        return


def main():
    parser = argparse.ArgumentParser(
        description='Entrypoint for Vikunja/Polybar integration.'
    )

    group = parser.add_mutually_exclusive_group()
    group.add_argument("--init", action="store_true")
    group.add_argument("--list-todo-lists", action="store_true")
    group.add_argument("--get-todo-count", action="store_true")
    group.add_argument("--list-todos", action="store_true")
    group.add_argument("--config-popup", action="store_true")
    group.add_argument("--show-todos-popup", action="store_true")

    args = parser.parse_args()

    if args.init:
        first_run()

    vikunja_client = PolybarVikunjaClient()

    if args.list_todo_lists:
        print(vikunja_client.list_todo_lists())
    elif args.get_todo_count:
        print(vikunja_client.get_todo_count())
    elif args.list_todos:
        print(vikunja_client.get_remaining_todos())
    elif args.config_popup:
        ConfigPopup(
            lists=vikunja_client.list_todo_lists()
        )
    elif args.show_todos_popup:
        RemainingTodosPopup(vikunja_client)

if __name__ == '__main__':
    main()



from .config_helper import PolybarVikunjaConfig
from tkinter import *


class ConfigPopup():

    def __init__(self, lists=None):
        self.root = Tk(className="polybar-vikunja")
        self.root.attributes('-type', 'dialog')
        self.config = PolybarVikunjaConfig()

        self.position_window()


        lists = [
            {
                "title": e["title"],
                "id": e["id"]
            } for e in lists
        ]

        listbox = Listbox(
            self.root,
            selectmode='SINGLE',
        )

        selected_index = None
        for i in range(len(lists)):
            e = lists[i]
            listbox.insert(e["id"], e["title"])

            if str(e["id"]) == self.config.get("default_list", None):
                selected_index = i

        listbox.selection_set(selected_index)
        listbox.see(selected_index)

        scrollbar = Scrollbar(self.root)
        listbox.config(yscrollcommand=scrollbar.set)
        scrollbar.config(command=listbox.yview)

        def handle_list_select(event):
            selected_index = listbox.curselection()[0]
            self.config.set(
                "default_list",
                str(lists[selected_index]["id"])
            )

        listbox.bind("<<ListboxSelect>>", handle_list_select)
        listbox.grid(row=0, column=0)

        self.attach_close_window_handler()
        self.root.mainloop()

    def position_window(self):
        width = 250
        height = 250

        x = self.root.winfo_pointerx()
        y = self.root.winfo_pointery()
        abs_coord_x = self.root.winfo_pointerx() - self.root.winfo_vrootx()
        abs_coord_y = self.root.winfo_pointery() - self.root.winfo_vrooty()

        abs_coord_x -= int(width / 2)
        abs_coord_y -= int(height) - int(height / 10)

        self.root.geometry(f"{width}x{height}+{abs_coord_x}+{abs_coord_y}")

    def attach_close_window_handler(self):
        def close_window(event=None):
            self.root.destroy()
        self.root.bind("<FocusOut>", close_window)

class RemainingTodosPopup():

    def __init__(self, vikunja_client):
        self.root = Tk(className="polybar-vikunja")
        self.root.attributes('-type', 'dialog')
        self.config = PolybarVikunjaConfig()
        self.vikunja_client = vikunja_client

        self.position_window()

        self.remaining_todos = [
            {
                "title": e["title"],
                "id": e["id"]
            } for e in vikunja_client.get_remaining_todos()
        ]

        self.is_checked = {
            todo["id"]: BooleanVar()
            for todo in self.remaining_todos
        }

        for i in range(len(self.remaining_todos)):
            row = Frame(self.root)
            row.pack(
                fill=BOTH,
                expand=True
            )

            Checkbutton(
                row,
                variable=self.is_checked[self.remaining_todos[i]["id"]],
                command=lambda todo_id=self.remaining_todos[i]["id"]: self.handle_checkbox_click(todo_id)
            ).pack(
                side=LEFT
            )
            Label(
                row,
                text=self.remaining_todos[i]["title"]
            ).pack(
                side=LEFT,
            )

        self.attach_close_window_handler()
        self.root.mainloop()

    def handle_checkbox_click(self, todo_id):
        self.vikunja_client.mark_todo_complete_status(
            todo_id,
            self.is_checked[todo_id].get()
        )

    def position_window(self):
        width = 350
        height = 300

        x = self.root.winfo_pointerx()
        y = self.root.winfo_pointery()
        abs_coord_x = self.root.winfo_pointerx() - self.root.winfo_vrootx()
        abs_coord_y = self.root.winfo_pointery() - self.root.winfo_vrooty()

        abs_coord_x -= int(width / 2)
        abs_coord_y -= int(height) - int(height / 10)

        self.root.geometry(f"{width}x{height}+{abs_coord_x}+{abs_coord_y}")

    def attach_close_window_handler(self):
        def close_window(event=None):
            self.root.destroy()
        self.root.bind("<FocusOut>", close_window)



import json
import os


class PolybarVikunjaConfig():

    def __init__(self, *args, **kwargs):
        self.config_file = kwargs.get(
            'config_file',
            '.polybar_vikunja_config.json'
        )
        self.path = os.path.expanduser("~") + "/" + self.config_file

    def get(self, key, default):
        if not os.path.exists(self.path):
            return default
        try:
            with open(self.path, 'r') as fh:
                config = json.load(fh)

                if key in config.keys():
                    return config[key]
                else:
                    return default
        except Exception as e:
            print(f"Problem reading from ~/{self.config_file}!")
            print(e)

    def set(self, key, value):
        config = None

        try:
            with open(self.path, 'r') as fh:
                config = json.load(fh)
        except Exception as e:
            print(f"Problem reading from ~/{self.config_file}!")
            print(e)

            return

        config[key] = value

        try:
            with open(self.path, 'w') as fh:
                json.dump(config, fh)
        except Exception as e:
            print(f"Problem writing to ~/{self.config_file}!")
            print(e)

            return
